from django.conf import settings

from xdj_datamap.defaults import TABLE_PREFIX


class Settings:

    @property
    def TABLE_PREFIX(self):
        return getattr(settings, "TABLE_PREFIX", TABLE_PREFIX)


conf = Settings()



# -*- coding: utf-8 -*-
from django.urls import re_path
from rest_framework import routers

from xdj_datamap.views.field import FieldsViewSet
from xdj_datamap.views.map import DataMapViewSet
from xdj_datamap.views.model import ModelViewSet

system_url = routers.SimpleRouter()
system_url.register(r'map', DataMapViewSet)
system_url.register(r'field', FieldsViewSet)
system_url.register(r'model', ModelViewSet)

urlpatterns = [
    re_path('map/get_data/(?P<datamap>.*?)/', DataMapViewSet.as_view({'get': 'get_data'})),
    re_path('map/get_model/(?P<datamap>.*?)/', DataMapViewSet.as_view({'get': 'get_model'})),
]
urlpatterns += system_url.urls



# -*- coding: utf-8 -*-

"""
@author: xuan
@contact: QQ:595127207
@Created on: 2021/6/1 001 22:38
@Remark: 数据映射模块
"""

from django.contrib.contenttypes.models import ContentType
from xdj_utils.serializers import CustomModelSerializer
from xdj_utils.viewset import CustomModelViewSet

from xdj_datamap.models import DataMapsModel


class ModelsSerializer(CustomModelSerializer):

    class Meta:
        model = ContentType
        fields = "__all__"



class ModelViewSet(CustomModelViewSet):

    queryset = DataMapsModel.objects.all()
    serializer_class = ModelsSerializer
    search_fields = ['app_label','model']


# -*- coding: utf-8 -*-

"""
@author: xuan
@contact: QQ:595127207
@Created on: 2021/6/1 001 22:38
@Remark: 数据映射模块
"""
import json
import logging

from django.apps import apps
from rest_framework import serializers
from xdj_utils.json_response import SuccessResponse
from xdj_utils.serializers import CustomModelSerializer
from xdj_utils.viewset import CustomModelViewSet

from xdj_datamap.models import DataMapsModel, FieldsModel

logger = logging.getLogger(__name__)


class FieldsSerializer(CustomModelSerializer):

    class Meta:
        model = FieldsModel
        fields = "__all__"


class DataMapSerializer(CustomModelSerializer):
    where_cluase = serializers.JSONField()
    fields = FieldsSerializer(many=True)

    class Meta:
        model = DataMapsModel
        fields = "__all__"


class DataMapViewSet(CustomModelViewSet):

    queryset = DataMapsModel.objects.all()
    serializer_class = DataMapSerializer
    search_fields = ['model_app', 'model_name', 'where_cluase', 'exclude_cluase']

    #获取datamap对象
    def get_obj(self, data):
        k = data
        datamap = k.get('datamap') or k.get('datamap_id')
        try:
            obj = self.queryset.get(id=datamap)
            return obj
        except DataMapsModel.DoesNotExist:
            logger.error(f'dose not have this datamap, datamap_id:{datamap}')

    #获取datamap对应的fields
    def get_fields_(self, obj):
        if obj is None: return []
        serializer = DataMapSerializer(obj)
        fields = serializer.data.get('fields')
        fields_ = []
        for field in fields:
            field = dict(field)
            fields_.append(field)
        fields_.sort(key=lambda o: o.get('field_sort'))

        return fields_

    # 获取datamap对应app和model的源模型，并按照where和exclude过滤
    def get_map_(self, obj):
        if obj is None: return self.queryset.none()

        model_app = obj.model_app
        model_name = obj.model_name
        where_cluase = json.loads(obj.where_cluase or "{}")
        exclude_cluase = json.loads(obj.exclude_cluase or "{}")
        qs = apps.get_model(model_app,model_name).objects.all()

        if qs is not None:
            qs = qs.filter(**where_cluase).exclude(**exclude_cluase)
        return qs

    def get_model_(self, request=None, *args, **kwargs):
        k = {**(request.data if request else {}), **kwargs}
        obj = self.get_obj(k)
        model_app = obj.model_app
        model_name = obj.model_name
        model = apps.get_model(model_app,model_name)
        model_ = []
        for m in model._meta.fields:
            model_.append({"name": m.name, "verbose_name": m.verbose_name})
        return model_

    def get_model(self, request, *args, **kwargs):
        return SuccessResponse(data=self.get_model_(request, args, kwargs), msg='获取成功')

    #获取数据
    def get_data_(self, request=None, *args, **kwargs):
        k = {**(request.data if request else {}), **kwargs}
        obj = self.get_obj(k)
        fields_ = self.get_fields_(obj)
        field_names = [o.get('field_name') for o in fields_]
        qs = self.get_map_(obj).values(*field_names).order_by(k.get('order_by','year'))
        ret = []
        for q in qs:
            ret.append(q)

        return {"body":ret,"fields":fields_}

    def get_data(self, request, *args, **kwargs):
        return SuccessResponse(data=self.get_data_(request, args, kwargs), msg='获取成功')


from django.db import models

from xdj_utils.models import CoreModel
from xdj_datamap.conf import conf

table_prefix = conf.TABLE_PREFIX


def default_dict():
    return {}


class DataMapsModel(CoreModel):
    where_cluase = models.CharField(max_length=255, blank=True, null=True)
    exclude_cluase = models.CharField(max_length=255, blank=True, null=True)
    model_app = models.CharField(max_length=255, blank=True, null=True)
    model_name = models.CharField(max_length=255, blank=True, null=True)
    #fields

    class Meta:
        db_table = table_prefix + "datamap_model"
        verbose_name = '数据映射表'
        verbose_name_plural = verbose_name
        ordering = ('create_datetime',)


class FieldsModel(CoreModel):
    datamap = models.ManyToManyField(to='DataMapsModel',related_name='fields', verbose_name="字段", null=True, blank=True,
                               db_constraint=False, help_text="字段")
    field_name = models.CharField(max_length=255, default="", blank=True, null=True, verbose_name="字段名")
    field_desc = models.CharField(max_length=255, default="", blank=True, null=True, verbose_name="字段描述")
    field_fath = models.CharField(max_length=255, default="", blank=True, null=True, verbose_name="字段父描述")
    field_unit = models.CharField(max_length=255, default="", blank=True, null=True, verbose_name="字段单位")
    field_show = models.CharField(max_length=255, default="", blank=True, null=True, verbose_name="字段表示方式")#%，整数，两位小数
    field_sort = models.IntegerField( default=0, verbose_name="字段排序")

    class Meta:
        db_table = table_prefix + "datamap_field"
        verbose_name = '字段表'
        verbose_name_plural = verbose_name
        ordering = ('create_datetime',)


# -*- coding: utf-8 -*-

"""
@author: xuan
@contact: QQ:595127207
@Created on: 2021/11/27 001 22:38
@Remark: 数据映射模块
"""

from xdj_utils.serializers import CustomModelSerializer
from xdj_utils.viewset import CustomModelViewSet

from xdj_datamap.models import FieldsModel


class FieldsSerializer(CustomModelSerializer):

    class Meta:
        model = FieldsModel
        fields = "__all__"


class FieldsViewSet(CustomModelViewSet):

    queryset = FieldsModel.objects.all()
    serializer_class = FieldsSerializer


#!/usr/bin/env python

import re
import setuptools

version = ""
with open('xdj_datamap/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="xdj-datamap",
    version=version,
    author="18580543261",
    author_email="595127207@qq.com",
    description="a datamap for django",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT Licence",
    url="https://gitee.com/xdjango/xdj_datamap.git",
    install_requires=[
        "Django==3.2.3",
        "djangorestframework==3.12.4",
        "xdj-utils==0.0.4"
    ],
    packages=setuptools.find_packages(),
    classifiers=(
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10"
    ),
    exclude_package_data={'': ["requirements.txt"]},
)
